﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Reflection;
using System.Web.UI;
using XlsxAspNet.Properties;

namespace XlsxAspNet
{
    public partial class XlsxAspNet : Page
    {
        private static string tempDir;
        private static string templateFile;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                tempDir = Path.Combine(this.Request.PhysicalApplicationPath, Settings.Default.TemporaryDirectory);
                templateFile = Path.Combine(this.Request.PhysicalApplicationPath, Settings.Default.TempateFilePath);

                var data = new DataTable(Settings.Default.DataTableName, Settings.Default.DataTableNamespace);
                using (var stream = Assembly.GetExecutingAssembly().GetManifestResourceStream("XlsxAspNet.XlsxReadWriteTable.xsd"))
                    data.ReadXmlSchema(stream);

                this.Session["data"] = data;

                // Set data binding.
                this.SetDataBinding();
            }
        }

        private void SetDataBinding()
        {
            this.gridView.DataSource = ((DataTable)Session["data"]).DefaultView;
            this.gridView.DataBind();
        }

        protected void ReadInput(object sender, EventArgs e)
        {
            // Input file name.
            string fileName = Path.Combine(this.Request.PhysicalApplicationPath, Settings.Default.InputFileName);

            // Delete contents of the temporary directory.
            XlsxRW.DeleteDirectoryContents(tempDir);

            // Unzip input XLSX file to the temporary directory.
            XlsxRW.UnzipFile(fileName, tempDir);

            IList<string> stringTable;
            // Open XML file with table of all unique strings used in the workbook..
            using (var stream = new FileStream(Path.Combine(tempDir, @"xl\sharedStrings.xml"),
                FileMode.Open, FileAccess.Read))
                // ..and call helper method that parses that XML and returns an array of strings.
                stringTable = XlsxRW.ReadStringTable(stream);

            // Get DataTable from session variable.
            var data = (DataTable)Session["data"];
            // Open XML file with worksheet data..
            using (var stream = new FileStream(Path.Combine(tempDir, @"xl\worksheets\sheet1.xml"),
                FileMode.Open, FileAccess.Read))
                // ..and call helper method that parses that XML and fills DataTable with values.
                XlsxRW.ReadWorksheet(stream, stringTable, data);

            // Update GridView.
            this.SetDataBinding();
        }

        protected void WriteOutput(object sender, EventArgs e)
        {
            // Output file name.
            string fileName = Path.Combine(this.Request.PhysicalApplicationPath, Settings.Default.OutputFileName);

            // Delete contents of the temporary directory.
            XlsxRW.DeleteDirectoryContents(tempDir);

            // Unzip template XLSX file to the temporary directory.
            XlsxRW.UnzipFile(templateFile, tempDir);

            // We will need two string tables; a lookup IDictionary<string, int> for fast searching and 
            // an ordinary IList<string> where items are sorted by their index.
            IDictionary<string, int> lookupTable;

            var data = (DataTable)Session["data"];

            // Call helper methods which creates both tables from input data.
            var stringTable = XlsxRW.CreateStringTables(data, out lookupTable);

            // Create XML file..
            using (var stream = new FileStream(Path.Combine(tempDir, @"xl\sharedStrings.xml"),
                FileMode.Create))
                // ..and fill it with unique strings used in the workbook
                XlsxRW.WriteStringTable(stream, stringTable);

            // Create XML file..
            using (var stream = new FileStream(Path.Combine(tempDir, @"xl\worksheets\sheet1.xml"),
                FileMode.Create))
                // ..and fill it with rows and columns of the DataTable.
                XlsxRW.WriteWorksheet(stream, data, lookupTable);

            // ZIP temporary directory to the XLSX file.
            XlsxRW.ZipDirectory(tempDir, fileName);

            if (this.downloadFileCheckBox.Checked)
            {
                // Stream XLS to browser.
                this.Response.Clear();
                this.Response.ContentType = "application/vnd.ms-excel";
                this.Response.AddHeader("Content-Disposition", "attachment; filename=Out.xlsx");
                this.Response.WriteFile(fileName);
                this.Response.End();
            }
        }
    }
}